##########################################################################################
##########################################################################################
####################### SIMULATING ALMA OBSERVATIONS OF AN M51 ANALOG ####################
##########################################################################################
##########################################################################################
# Written by K. Nyland for the Cycle 7 ALMA proposal workshop held at UMD
# Date: 3/21/2019
# Goal: Simulate ALMA observations of a complex source with the 12m array + ACA (7m + TP)
# Tasks: SIMOBSERVE, SIMANALYZE
# NOTE 1: Although this example simulates a "continuum" mosaicking observation with the 
#         12m, 7m, & TP arrays, TP mosaicking in continuum mode is not allowed in Cycle 7. 
#         Cycle 7 does, however, permit TP mosaicking in spectral line mode.  The example 
#         given in this script (and on the CASA Guides) should thus be interpreted as 
#         illustrative only. Future ALMA proposal cycles are expected to include additional  
#         TP observing modes.
# NOTE 2: This example is loosely based on the M51 ACA simulation CASA Guide in the link 
#         below. Be aware that numerous changes have intentionally been made here compared 
#         to the CASA Guide version (as of 3/21/19) to improve the consistency & clarity.
#         CASA Guide version: https://casaguides.nrao.edu/index.php/ACA_Simulation_(CASA_5.4)
# NOTE 3: Remember that simulations are not required for ALMA proposals, but may in some 
#         cases help strengthen them.
##########################################################################################
import os

# Download model image
if not os.path.exists('M51ha.fits'):
	os.system('curl https://casaguides.nrao.edu/images/3/3f/M51ha.fits.txt -f -o M51ha.fits')

# View available antenna configurations in CASA
mydir = os.getenv('CASAPATH').split()[0]+'/data/alma/simmos/'
os.system('ls ' + mydir)

##########################################################################################
######################################## SIMOBSERVE ######################################
##########################################################################################

############################## SIMULATE 12m ARRAY OBSERVATIONS ###########################

# Set SIMOBSERVE to default parameters
default(simobserve)

# All simulation products will be placed in a subdirectory called M51_simulation
project            = 'M51_simulation'

# Set model image (M51ha.fits is an Halpha image of M51 with re-scaled pixels)
skymodel           = 'M51ha.fits'

# Set model image parameters:
indirection        = "J2000 23h59m59.96s -34d59m59.50s"
incell             = "0.1arcsec"
inbright           = "0.004"
incenter           = "345GHz"
inwidth            = "7.5GHz"

# Let SIMOBSERVE calculate mosaic pointing locations:
setpointings       = True
integration        = "10s"
mapsize            = "1arcmin"
maptype            = "hex"
pointingspacing    = '' # leave blank for auto calculation of pointings by simobserve

# Let SIMOBSERVE select config based on input resolution
obsmode            = "int"
antennalist        = "ALMA;0.5arcsec" # can also specify "alma.cycle6.3.cfg"
totaltime          = "1h"

# Run simobserve
graphics           = "both"
simobserve()

############################### SIMULATE 7m ACA OBSERVATIONS #############################
tget(simobserve)
integration        = "10s"
mapsize            = "1arcmin"
maptype            = "hex"
pointingspacing    = '' # we will let SIMOBSERVE choose the pointing spacing
obsmode            = "int"
refdate            = "2012/12/02" # NOTE: change the date from 12m array sims
antennalist        = "aca.i.cfg"
totaltime          = "2.4h"
simobserve()

# QUESTION: How does the diagnostic output from SIMOBSERVE for the 7m ACA simulation 
#           compare to the simulated 12m array output?

############################# SIMULATE 12m TP ACA OBSERVATIONS ###########################
tget(simobserve)
integration        = "10s"
mapsize            = "1.3arcmin"
maptype            = "hex"
obsmode            = "sd"
sdantlist          = "aca.tp.cfg"
refdate            = "2012/12/01" # NOTE: change the date from 7m and 12m array sims
totaltime          = "4.1h"
simobserve()

##########################################################################################
######################################## SIMANALYZE ######################################
##########################################################################################

############################## RUN SIMANALYZE ON TP + 7m ACA #############################
default(simanalyze)
project            = "M51_simulation"
vis                = '$project.aca.i.noisy.ms,$project.aca.tp.noisy.sd.ms'  
imsize             = [400,400]
cell               = '0.2arcsec'
threshold          = '0.3mJy'
niter              = 10000
pbcor              = True
analyze            = True
showuv             = True
showpsf            = False
showmodel          = True
showconvolved      = True
showclean          = True
showresidual       = False
showdifference     = True
verbose            = True
showfidelity       = True
simanalyze()

# NOTE: The clean threshold is set to slightly higher than the sensitivity of the ACA data 
#       from the online sensitivity calculator

############################### RUN SIMANALYZE ON 12m ARRAY ##############################
default("simanalyze")
project            = "M51_simulation"
vis                = '$project.ALMA_0.5arcsec.noisy.ms'
modelimage         = "$project.aca.i.noisy.feather.image" # simanalyze output of ACA data (7m + TP)
imsize             = [600,600]
cell               = '0.1arcsec'
threshold          = '0.028mJy'
niter              = 10000
pbcor              = True
analyze            = True
showuv             = True
showpsf            = False
showmodel          = True
showconvolved      = True
showclean          = True
showresidual       = False
showdifference     = True
verbose            = True
showfidelity       = True
simanalyze()

# NOTE: The clean threshold is set to slightly higher than the sensitivity of the 12m  
#       array from the online sensitivity calculator

# QUESTION: How does the diagnostic output from SIMANALYZE for the ACA data compare to 
#           the 12m array output?  Compare the *.image.png and *.analysis.png files.  You 
#           can also view the output data products from SIMANALYZE in the CASA Viewer.  
#           See TCLEAN tutorial for additional details on using the Viewer (or visit 
#           https://casa.nrao.edu/casadocs/casa-5.4.0/image-cube-visualization/viewer-basics)

