##########################################################################################
##########################################################################################
############### IMAGING SIMULATED 12m ARRAY ALMA OBSERVATIONS OF AN M51 ANALOG ###########
##########################################################################################
##########################################################################################
# Written by K. Nyland for the Cycle 7 ALMA proposal workshop held at UMD
# Date: 3/21/2019
# Goal: image simulated ALMA data using TCLEAN
# Tasks: TCLEAN, IMVIEW/VIEWER, EXPORTFITS
##########################################################################################
import os

# Set project name
project            = 'M51_simulation'

# Set center of map (indirection from simobserve)
indirection        = "J2000 23h59m59.96s -34d59m59.50s"

# Set simulated 12m array measurement set name
msfile = os.path.join(project, project + '.ALMA_0.5arcsec.noisy.ms')

# Check to see if 12m array msfile and ACA image are in working directory
if not os.path.exists(msfile):
	print 'Missing measurement set - check working directory'
	
##########################################################################################
######################### IMAGE 12m ARRAY SIMULATION - SINGLE SCALE ######################
##########################################################################################

# Run tclean with natural weighting and standard single-scale (hogbom) deconvolution
imversion          = '1' # for keeping track of different trial image versions 
default(tclean)
vis                = msfile
phasecenter        = indirection
imsize             = [600,600]
cell               = '0.1arcsec'
gridder            = 'mosaic'
usepointing        = True
weighting          = 'natural'
deconvolver        = 'hogbom'
niter              = 10000
threshold          = '0.028mJy'
interactive        = False
outname            = vis.split('.ms')[0] + '_' + 'tclean_12m_array_' + weighting + '_' + deconvolver + '_v' + imversion
imagename          = outname
tclean()

# Inspect image (the image suffix with devonvolver = 'hogbom' will be '.image')
default(imview)
raster = outname + '.image'
imview()

# Either from within the Viewer GUI or with a separate call to imview, inspect the other 
# image products (the .residual and .model are particularly important to inspect)

# Run exportfits (for viewing/analyzing the image in ds9, Python, etc.)
default(exportfits)
imagename = outname + '.image'
fitsimage = outname + '.fits'
exportfits()

##########################################################################################
########################## IMAGE 12m ARRAY SIMULATION - MULTISCALE #######################
##########################################################################################

# Set parameters and run tclean with natural weighting and multiscale deconvolution
default(tclean)
imversion          = '1' # for keeping track of different trial image versions 
vis                = msfile
phasecenter        = indirection
imsize             = [600,600]
cell               = '0.1arcsec'
gridder            = 'mosaic'
usepointing        = True
weighting          = 'natural'
deconvolver        = 'mtmfs'
nterms             = 1
scales             = [0,5,15]
niter              = 10000
threshold          = '0.028mJy'
interactive        = False
outname            = vis.split('.ms')[0] + '_' + 'tclean_12m_array_' + weighting + '_' + deconvolver + '_v' + imversion
imagename          = outname
tclean()

# Inspect image (the image suffix with devonvolver = 'mtmfs' will be '.image.tt0')
default(imview)
raster = outname + '.image.tt0'
imview()

# Either from within the Viewer GUI or with a separate call to imview, inspect the other 
# image products (the .residual.tt0 and .model.tt0 are particularly important to inspect)

# Run exportfits (for viewing/analyzing the image in ds9, Python, etc.)
default(exportfits)
imagename = outname + '.image.tt0'
fitsimage = outname + '.fits'
exportfits()

##########################################################################################
################################# QUESTIONS TO THE STUDENT ###############################
##########################################################################################

# 1) How does the image change when Briggs weighting is used (weighting='briggs')?

# 2) How do the image and residual look when niter=0 ("dirty image")?

# 3) What happens when you increase the threshold parameter?

# 4) How does imaging with interactive = True and drawing clean masks alter the result?

# 5) How does increasing the map sizes in SIMOBSERVE and imaging with a larger imsize 
#    in TCLEAN impact the image quality?

##########################################################################################
############################ EXERCISES FOR THE ADVANCED STUDENT ##########################
##########################################################################################

# 1) Try imaging the noisy 12m & 7m array interferometric datasets together.  To do this, 
#    you will need to first combine the datasets using the task CONCAT with the 
#    visweightscale parameter set to an appropriate value. The Band 3 M100 CASA Guide 
#    (https://casaguides.nrao.edu/index.php/M100_Band3_Combine_5.4) discusses this topic. 
#    Next, image the combined output dataset from CONCAT in TCLEAN.  How does this image 
#    compare to the image of just the 12m array?  How do the residual images compare?

# 2) Once you are happy with your combined 12m + 7m array image, try feathering it with 
#    the TP image produced by SIMANALYZE.  The Band 3 M100 Guide in the link above also 
#    provides an example of feathering.
 